#include "stdafx.hpp"
#include "resource.h"
#include "FWatchApp.hpp"
#include "SettingDlg.hpp"

#include "Utility.hpp"
#include "tstringUty.hpp"

#include <assert.h>

#include <shlobj.h>

#include <string>
#include <vector>

tstring WindowUtility::getWindowText( HWND v_hWnd ) const throw()
{
	//required:
	assert( v_hWnd != NULL && "v_hWndNULL͎wł܂B" );

	//do:
	const int len = ::GetWindowTextLength( v_hWnd );
	if( len > 0 ) {
		std::vector< tstring::value_type > buffer( len + 1 );
		::GetWindowText( v_hWnd, &buffer[0], len + 1 );
		return tstring( &buffer[0] );
	}
	return "";
}

void WindowUtility::setWindowText( HWND v_hWnd, const tstring& v_text ) const throw()
{
	//required:
	assert( v_hWnd != NULL && "v_hWndNULL͎wł܂B" );

	//do:
	::SetWindowText( v_hWnd, v_text.c_str() );
}

//////////////

CSettingDlg::CSettingDlg( const CSettingInfo& v_settingInfo )
	: hDlg_( NULL )
	, settingInfo_( v_settingInfo )
	, factory_( v_settingInfo )
	, errorOccured_( false )
{
}

CSettingDlg::~CSettingDlg()
{
}

bool CSettingDlg::show( HWND hWnd )
{
	return DialogBoxParam(
		pApp->getInstanceHandle(),
		(LPCTSTR)IDD_SELDIRDLG,
		hWnd,
		(DLGPROC)DlgProc,
		(LPARAM)this
		) == IDOK;
}

/*!
 * t@Ngp[^̐ݒɉ߂邽тɃG[ʒm܂B
 * ߂lƂfalsew肷ƃp[^͐ݒ肳܂B
 * truew肷ƁA񕜂łꍇ͉񕜂ăp[^ݒ肳܂B
 * Ap[^̉񕜂łȂꍇfalseƓłB
 * \param v_errorInfo G[
 * \return 񕜂ꍇtruew肵܂B
 */
bool CSettingDlg::notifyErrorInfo( const tstring& v_errorInfo ) throw()
{
	const tstring errorCaption = pApp->LoadString( IDS_ERROR_SETTING );
	errorOccured_ = true;
	::MessageBox( hDlg_, v_errorInfo.c_str(), errorCaption.c_str(), MB_ICONERROR | MB_OK );
	return false;
}

CSettingInfo CSettingDlg::createSettingInfo() const
{
	try{
		return factory_.create();
	}
	catch( const CSettingInfoFactory::CSettingInfoFactoryException& ) {
		assert( false && "p[^̍\zɎs܂Bshow\bhIɌĂяo\܂B" );
		throw;
	}
}

void CSettingDlg::OnInitDialog()
{
	hDir  = GetDlgItem( hDlg_, IDC_WATCHDIR );
	assert( hDir != NULL );
	hName = GetDlgItem( hDlg_, IDC_FILETYPE );
	assert( hName != NULL );
	hVerb = GetDlgItem( hDlg_, IDC_ACTION );
	assert( hVerb != NULL );
	hApp  = GetDlgItem( hDlg_, IDC_APPNAME );
	assert( hApp != NULL );
	hParam= GetDlgItem( hDlg_, IDC_PARAM );
	assert( hParam != NULL );
	hShowWindow = GetDlgItem( hDlg_, IDC_WINDOWSIZE );
	assert( hShowWindow != NULL );
	hWaitWrite = GetDlgItem( hDlg_, IDC_WAITWRITABLE );
	assert( hWaitWrite != NULL );
	hTolerance = GetDlgItem( hDlg_, IDC_TOLERANCE );
	assert( hTolerance != NULL );
	hDeletePending = GetDlgItem( hDlg_, IDC_DELETEPENDING );
	assert( hDeletePending != NULL );
	hLog = GetDlgItem( hDlg_, IDC_LOGFILE);
	assert( hLog != NULL );
	hMaxProcess = GetDlgItem( hDlg_, IDC_MAXPROCESS );
	assert( hMaxProcess != NULL );

	hMaxDepth = GetDlgItem( hDlg_, IDC_MAXDEPTH );
	assert( hMaxDepth != NULL );
	hDirNotificationAPIRetrySpan = GetDlgItem( hDlg_, IDC_DIRNOTIFICATIONAPIRETRYSPAN );
	assert( hDirNotificationAPIRetrySpan != NULL );
	hDirNotificationAPIExpirySpan = GetDlgItem( hDlg_, IDC_DIRNOTIFICATIONAPIRETRYEXPIRYSPAN );
	assert( hDirNotificationAPIExpirySpan != NULL );
	hForceInterval = GetDlgItem( hDlg_, IDC_FORCEINTERVAL );
	assert( hForceInterval != NULL );

	hDelaySignalSpan = GetDlgItem( hDlg_, IDC_DELAYSIGNALSPAN );
	assert( hDelaySignalSpan != NULL );
	hAppCurrentDir = GetDlgItem( hDlg_, IDC_APPCURRENTDIR );
	assert( hAppCurrentDir != NULL );

	// EBhETCY̑I
	SendMessage( hShowWindow, CB_INSERTSTRING, 0, (LPARAM) ShowWindowType::valueOf( 0 ).getDescription().c_str() );
	SendMessage( hShowWindow, CB_INSERTSTRING, 1, (LPARAM) ShowWindowType::valueOf( 1 ).getDescription().c_str() );
	SendMessage( hShowWindow, CB_INSERTSTRING, 2, (LPARAM) ShowWindowType::valueOf( 2 ).getDescription().c_str() );
	SendMessage( hShowWindow, CB_INSERTSTRING, 3, (LPARAM) ShowWindowType::valueOf( 3 ).getDescription().c_str() );
	SendMessage( hShowWindow, CB_INSERTSTRING, 4, (LPARAM) ShowWindowType::valueOf( 4 ).getDescription().c_str() );

	// ANV̑I
	const tstring verbList = pApp->LoadString( IDS_SETTINGMES_VERBLIST );
	
	tstring::size_type lastpos = 0;
	for(;;) {
		tstring::size_type pos = verbList.find( '|', lastpos );
		tstring verb;
		if( pos != tstring::npos ) {
			verb = verbList.substr( lastpos, pos - lastpos );
		}
		else {
			verb = verbList.substr( lastpos );
		}
		SendMessage( hVerb, CB_INSERTSTRING, -1, (LPARAM) verb.c_str() );

		if( pos == tstring::npos ) {
			break;
		}
		lastpos = pos + 1;
	}

	setInitialParam();

	// t@Ng̃G[ʒmÃ_CAO󂯂܂B
	factory_.setErrorInfoListener( this );
}

void CSettingDlg::setInitialParam()
{
	SendMessage( hShowWindow, CB_SETCURSEL, settingInfo_.getShowWindow(), 0 );

	setWindowText( hVerb, settingInfo_.getAction() );
	setWindowText( hDir, settingInfo_.getWatchDir() );
	setWindowText( hName, settingInfo_.getWatchFile() );
	setWindowText( hApp, settingInfo_.getAppName() );
	setWindowText( hParam, settingInfo_.getParam() );
	setWindowText( hLog, settingInfo_.getLog() );

	setWindowText( hTolerance, tstringuty::IntToString( settingInfo_.getTolerance() ) );
	setWindowText( hWaitWrite, tstringuty::IntToString( settingInfo_.getWaitWrite() ) );

	setWindowText( hDeletePending, tstringuty::IntToString( settingInfo_.getDeletePending() ) );
	setWindowText( hMaxProcess, tstringuty::IntToString( settingInfo_.getMaxProcess() ) );

	setWindowText( hMaxDepth, tstringuty::IntToString( settingInfo_.getMaxDepth() ) );

	CheckDlgButton( hDlg_, IDC_USEDIRNITIFICATIONAPI, settingInfo_.isUsingDirNotificationAPI() ? BST_CHECKED : BST_UNCHECKED );

	setWindowText( hDirNotificationAPIRetrySpan, tstringuty::IntToString( settingInfo_.getDirNotificationAPIRetryInterval() ) );
	setWindowText( hDirNotificationAPIExpirySpan, tstringuty::IntToString( settingInfo_.getDirNotificationAPIExpirySpan() ) );
	setWindowText( hForceInterval, tstringuty::IntToString( settingInfo_.getForceInterval() ) );

	setWindowText( hDelaySignalSpan, tstringuty::IntToString( settingInfo_.getDelaySignalSpan() ) );
	setWindowText( hAppCurrentDir, settingInfo_.getAppCurrentDir() );
}

void CSettingDlg::onOK()
{
	clearError();

	factory_.setWatchDir( getWindowText( hDir ) );
	factory_.setWatchFile( getWindowText( hName ) );
	factory_.setAction( getWindowText( hVerb ) );
	factory_.setAppName( getWindowText (hApp ) );
	factory_.setParam( getWindowText( hParam ) );
	factory_.setLog( getWindowText( hLog ) );
	factory_.setWaitWrite( tstringuty::atoi( getWindowText( hWaitWrite ) ) );
	factory_.setTolerance( tstringuty::atoi( getWindowText( hTolerance ) ) );

	factory_.setDeletePending( tstringuty::atoi( getWindowText( hDeletePending ) ) );
	factory_.setMaxProcess( tstringuty::atoi( getWindowText( hMaxProcess ) ) );

	factory_.setAppCurrentDir( getWindowText( hAppCurrentDir ) );
	factory_.setDelaySignalSpan( tstringuty::atoi( getWindowText( hDelaySignalSpan ) ) );

	const int forceInterval = tstringuty::atoi( getWindowText( hForceInterval ) );
	if( forceInterval > 0 ) {
		factory_.setForceInterval( forceInterval );
		factory_.setUsingDirNotificationAPI( ( IsDlgButtonChecked( hDlg_, IDC_USEDIRNITIFICATIONAPI ) == BST_CHECKED ) ? true : false );
	}
	else {
		factory_.setUsingDirNotificationAPI( ( IsDlgButtonChecked( hDlg_, IDC_USEDIRNITIFICATIONAPI ) == BST_CHECKED ) ? true : false );
		factory_.setForceInterval( forceInterval );
	}

	factory_.setDirNotificationAPIRetryInterval( tstringuty::atoi( getWindowText( hDirNotificationAPIRetrySpan ) ) );
	factory_.setDirNotificationAPIExpirySpan( tstringuty::atoi( getWindowText( hDirNotificationAPIExpirySpan ) ) );

	factory_.setMaxDepth( tstringuty::atoi( getWindowText( hMaxDepth ) ) );

	LRESULT nShowWindow;
	if( ( nShowWindow = SendMessage( hShowWindow, CB_GETCURSEL, 0, 0 ) ) == CB_ERR ) {
		nShowWindow = 1;
	}
	factory_.setShowWindow( ShowWindowType::valueOf( (int) nShowWindow ) );

	if( ! isErrorOccured() && factory_.validate( false ) ) {
		EndDialog(hDlg_,IDOK);
	}
}

void CSettingDlg::onCancel()
{
	EndDialog(hDlg_,IDCANCEL);
}

void CSettingDlg::onBrowseDir()
{
	IMalloc* g_pMalloc = NULL;
	if( FAILED( ::SHGetMalloc( &g_pMalloc ) ) ) {
		pApp->showMessageBox( MB_ICONERROR | MB_OK, NULL, IDS_MAINPROC_SHELLALLOCERROR );
		return;
	}

	const tstring caption = pApp->LoadString( IDS_SETTINGMES_SELECTDIR );
	
	TCHAR dirname[MAX_PATH];

	BROWSEINFO bi = {0};
	bi.hwndOwner = hDlg_;
	bi.pidlRoot = NULL;
	bi.pszDisplayName = dirname;
	bi.lpszTitle = caption.c_str();
	bi.ulFlags = BIF_RETURNONLYFSDIRS;
	bi.lpfn = NULL;
	bi.lParam = NULL;
	bi.iImage = 0;
	LPITEMIDLIST pidlist = ::SHBrowseForFolder( &bi );
	if( pidlist ){
		if( ::SHGetPathFromIDList( pidlist, dirname ) ) {
			TrimSpaceBothBuff( dirname );
			EnsureEndsWithBuff( dirname, "\\" );
			setWindowText(hDir, dirname );
		}
		g_pMalloc->Free( pidlist );
	}
	g_pMalloc->Release();
}

void CSettingDlg::onOpen()
{
	IMalloc* g_pMalloc = NULL;
	if( FAILED( ::SHGetMalloc( &g_pMalloc ) ) ) {
		pApp->showMessageBox( MB_ICONERROR | MB_OK, NULL, IDS_MAINPROC_SHELLALLOCERROR );
		return;
	}
	TCHAR InitDir[MAX_PATH] = { 0 };
	LPITEMIDLIST itemidlist;
	if( ::SHGetSpecialFolderLocation( hDlg_, CSIDL_STARTMENU, &itemidlist ) == NOERROR ) {
		::SHGetPathFromIDList( itemidlist, InitDir );
		g_pMalloc->Free(itemidlist);
	}
	g_pMalloc->Release();

	tstring filter = pApp->LoadString( IDS_SETTINGMES_OPENFILEFILTER );
	tstring::size_type pos;
	while( ( pos = filter.find( '|' ) ) != tstring::npos ) {
		filter.replace( pos, 1, 1, '\0' );
	}
	filter.push_back( '\0' );

	TCHAR openname[MAX_PATH] = { 0 };
	OPENFILENAME ofn = {0};
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.hwndOwner = hDlg_;
	ofn.hInstance = pApp->getInstanceHandle();
	ofn.lpstrFilter = filter.data();
	ofn.lpstrCustomFilter = NULL;
	ofn.nMaxCustFilter = 0;
	ofn.nFilterIndex = 0;
	ofn.lpstrFile = openname;
	ofn.nMaxFile = MAX_PATH;
	ofn.lpstrInitialDir = InitDir;
	ofn.lpstrTitle = NULL;
	ofn.Flags = OFN_FILEMUSTEXIST|OFN_HIDEREADONLY;
	if( ::GetOpenFileName( &ofn ) ) {
		setWindowText( hApp, openname );
	}
}

void CSettingDlg::onLogSelect()
{
	tstring filter = pApp->LoadString( IDS_SETTINGMES_LOGFILEFILTER );
	tstring::size_type pos;
	while( ( pos = filter.find( '|' ) ) != tstring::npos ) {
		filter.replace( pos, 1, 1, '\0' );
	}
	filter.push_back( '\0' );

	TCHAR savename[MAX_PATH] = { 0 };
	OPENFILENAME ofn = {0};
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.hwndOwner = hDlg_;
	ofn.hInstance = pApp->getInstanceHandle();
	ofn.lpstrFilter = filter.data();
	ofn.lpstrCustomFilter = NULL;
	ofn.nMaxCustFilter = 0;
	ofn.nFilterIndex = 0;
	ofn.lpstrFile = savename;
	ofn.nMaxFile = MAX_PATH;
	ofn.lpstrInitialDir = NULL;
	ofn.lpstrTitle = NULL;
	ofn.Flags = OFN_FILEMUSTEXIST;
	ofn.lpstrDefExt = "log";
	if( ::GetSaveFileName( &ofn ) ){
		setWindowText( hLog, savename );
	}
}

BOOL CSettingDlg::onCommand( UINT btn )
{
	switch(btn)
	{
	case IDOK:
		onOK();
		break;
		
	case IDCANCEL:
		onCancel();
		break;

	case IDC_BROWSE:
		onBrowseDir();
		break;

	case IDC_OPEN:
		onOpen();
		break;

	case IDC_LOGFILESELECT:
		onLogSelect();
		break;
	
	default:
		return FALSE;
	}
	
	return TRUE;
}

LRESULT CALLBACK CSettingDlg::DlgProc( HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam )
{
	CSettingDlg* me = (CSettingDlg*) ::GetWindowLongPtr( hDlg, DWLP_USER );
	assert( ( me == NULL || me->hDlg_ == hDlg ) && "QƂsłB" );

	if( message == WM_INITDIALOG ) {
		me = (CSettingDlg*) lParam;
		assert( me != NULL && "CX^Xւ̃|C^ݒ肳Ă܂B" );
		::SetWindowLongPtr( hDlg, DWLP_USER, (LONG) me );
		me->hDlg_ = hDlg;
		me->OnInitDialog();
		return TRUE;
	}
	else if( message == WM_COMMAND ) {
		return me->onCommand( LOWORD(wParam) );
	}

	return FALSE;
}

