#include "stdafx.hpp"

#include "FWatchApp.hpp"
#include "resource.h"

#include "Utility.hpp"
#include "tstringUty.hpp"
#include "DelaySignal.hpp"
#include "PassiveTimer.hpp"

#include "WatchThread.hpp"

#include <process.h>
#include <assert.h>

#include <string>

CWatchThread::CWatchThread( const CSettingInfo& v_settingInfo )
	: settingInfo_( v_settingInfo )
	, fileTimeComparator_( v_settingInfo.getTolerance() )
	, nThreadID_( 0 )
	, hThread_( NULL )
	, nameFilter_( false )
{
	hStopEvent_ = ::CreateEvent( NULL, TRUE, FALSE, NULL );
	assert( hStopEvent_ != NULL && "Cxgnh擾ł܂B" );

	const ActionInvokerFactory actionInvokerFactory( settingInfo_ );
	pActionInvoker_ = actionInvokerFactory.create();

	const tstring nameMatches = settingInfo_.getWatchFile();
	const tstring::size_type len = nameMatches.length();
	tstring::size_type idx = 0;
	tstring::size_type lastIdx = 0;
	for(;;) {
		const tstring::value_type ch = ( idx < len ) ? nameMatches[ idx ] : 0;
		if( ch == ';' || ch == 0 ) {
			const tstring nameMatch = tstringuty::trimSpaceBoth( nameMatches.substr( lastIdx, idx - lastIdx ) );
			if( ! nameMatch.empty() ) {
				nameFilter_.addPattern( SimplePatternMatch( nameMatch ) );
			}
			lastIdx = idx + 1;
		}
		if( idx >= len ) {
			break;
		}
#ifndef _UNICODE
		if( ::IsDBCSLeadByte( ch ) ) {
			idx++;
		}
#endif
		idx++;
	}
}


CWatchThread::~CWatchThread()
{
	Stop();
	::CloseHandle( hStopEvent_ );
}

const CSettingInfo& CWatchThread::getSettingInfo() const
{
	return settingInfo_;
}


bool CWatchThread::isRunning() const
{
	if( hThread_ == NULL ) {
		return false;
	}
	return ( ::WaitForSingleObject( hThread_, 0 ) == WAIT_TIMEOUT );
}

void CWatchThread::StopNoWait()
{
	if( hThread_ == NULL ) {
		return;
	}

	::SetEvent( hStopEvent_ );
}

void CWatchThread::Stop()
{
	if( hThread_ == NULL ) {
		return;
	}

	StopNoWait();
	::WaitForSingleObject( hThread_, INFINITE );
	hThread_ = NULL;
}

void CWatchThread::Start()
{
	//required:
	assert( hThread_ == NULL && "Xbh͂łɊJnĂ܂B" );

	//do:
	Stop();

	::ResetEvent( hStopEvent_ );
	hThread_ = (HANDLE) _beginthreadex( NULL, 0, FWatchThread, this, CREATE_SUSPENDED, (unsigned int*) &nThreadID_ );
	if( hThread_ == NULL ) {
		assert( false && "Xbh̊JnɎs܂B" );
		return; //TODO: O
	}
	::ResumeThread( hThread_ );
}

bool CWatchThread::getFileSize( const tstring& v_path, DWORD* v_pSizeLow, DWORD* v_pSizeHi )
{
	//required:
	assert( ! v_path.empty() && "t@Cɋ󕶎͎wł܂B" );
	
	//do:
	HANDLE fileHandle = ::CreateFile( v_path.c_str(), GENERIC_READ, FILE_SHARE_READ|FILE_SHARE_WRITE,NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );
	if( fileHandle != INVALID_HANDLE_VALUE ) {
		if( v_pSizeLow != NULL ) {
			*v_pSizeLow = ::GetFileSize( fileHandle, v_pSizeHi );
		}
		::CloseHandle( fileHandle );
		return true;
	}
	return false;
}

void CWatchThread::EnumFile( const bool v_enableEvent )
{
	// 폜ۗ̊؂ACȅ
	sweepDeletedEntries();

	// t@C̑
	if( EnumFileRecursive( settingInfo_.getWatchDir(), 0, v_enableEvent ) ) {
		// 폜ΏۃACe̐ݒ
		// JnfBNg̃t@C񋓂ꍇ̂ݍ폜ҋ@XgZbg܂B
		prepareDeleteEntries();
	}
}

void CWatchThread::prepareDeleteEntries()
{
	const DWORD now = ::GetTickCount();
	deletePendingList_.clear();
	for( CFileInfoMap::iterator ite_fileInfo = fileInfoMap_.begin();
		ite_fileInfo != fileInfoMap_.end(); )
	{
		const tstring& absolutePath = ite_fileInfo->first;
		CFileInfo& fileInfo = ite_fileInfo->second;
		if( ! fileInfo.isExist() ) {
			if( settingInfo_.getDeletePending() > 0 ) {
				// 폜ҋ@
				fileInfo.setDeletePending( true );
				fileInfo.setDeleteBeginTickCount( now );
				deletePendingList_.push_back( absolutePath );
			}
			else {
				// 폜ҋ@Ȃ
				ite_fileInfo = fileInfoMap_.erase( ite_fileInfo );
				continue;
			}
		}
		++ite_fileInfo;
	}
}

void CWatchThread::sweepDeletedEntries()
{
	unsigned int pendingCount = 0;

	const DWORD now = ::GetTickCount();
	const DWORD pendingLimit = (DWORD) ( settingInfo_.getDeletePending() * 100 );

	for( FileNameList::iterator ite_pendingList = deletePendingList_.begin();
		ite_pendingList != deletePendingList_.end(); )
	{
		const tstring& absoluteName = *ite_pendingList;
		CFileInfoMap::iterator ite_fileInfo = fileInfoMap_.find( absoluteName );
		
		bool removed = false;
		if( ite_fileInfo == fileInfoMap_.end() ) {
			assert( false && "폜ҋ@XgɁAt@CGg}bvɑ݂Ȃt@Cw肳Ă܂B" );
			removed = true;
		}
		else {
			CFileInfo& fileInfo = ite_fileInfo->second;
			if( fileInfo.isExist() ) {
				// t@Ĉ݂ō폜ҋ@Xg珜
				removed = true;
			}
			else if( fileInfo.isDeletePending() ) {
				if( ( settingInfo_.getDeletePending() <= 0 ) ||
					( ( now - fileInfo.getDeleteBeginTickCount() ) > pendingLimit )	)
				{
					// 폜ҋ@ΏۂłA폜Aw莞Ԃo߂Ă̂
					// t@CGg}bvƁA폜ҋ@Xg̑o폜B
					fileInfoMap_.erase( ite_fileInfo );
					removed = true;
				}
			}
		}

		if( removed ) {
			ite_pendingList = deletePendingList_.erase( ite_pendingList );
			continue;
		}
		++ite_pendingList;
	}
}

bool CWatchThread::EnumFileRecursive( const tstring& v_baseDir, const int v_depth, const bool v_enableEvent )
{
	// ΏۃpX𐶐
	const tstring baseDir = tstringuty::ensureEndsWith( v_baseDir, tstring("\\") );
	
	// ύXƎvt@C
	WIN32_FIND_DATA finddata = { 0 };
	HANDLE findHandle = ::FindFirstFile( ( baseDir + "*.*" ).c_str(), &finddata );
	if( findHandle == INVALID_HANDLE_VALUE ) {
		// ΏۂɃANZXs\ȂΉȂŖ߂B
		return false;
	}

	if( v_depth == 0 ) {
		// ݂Ȃt@C폜邽߁AׂẴt@C݃tOZbgB
		// (lbg[N̈ꎞIȐؒf̉\̂߁AΏۂɃANZXłꍇ̂ݎsB)
		for( CFileInfoMap::iterator ite = fileInfoMap_.begin();
			ite != fileInfoMap_.end();
			++ite)
		{
			CFileInfo& fileInfo = ite->second;
			fileInfo.setExist( false );
		}
	}

	const DWORD now = ::GetTickCount();
	for(;;) {
		const tstring absolutePath = baseDir + finddata.cFileName;
		if( finddata.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY ) {
			if( ! IsMatch( finddata.cFileName, "." ) && ! IsMatch( finddata.cFileName, ".." ) ) {
				if( v_depth < settingInfo_.getMaxDepth() ) {
					EnumFileRecursive( absolutePath, v_depth + 1, v_enableEvent ); // TufBNǧs̗L͕s
				}
			}
		}
		else {
			if( nameFilter_.isMatch( finddata.cFileName ) ) {
				CFileInfoMap::iterator ite = fileInfoMap_.find( absolutePath );
				if( ite != fileInfoMap_.end() ) {
					CFileInfo& fileInfo = ite->second;
					if( v_enableEvent && ! fileTimeComparator_( finddata.ftLastWriteTime, fileInfo.getLastModified() ) ) {
						fileInfo.setDetectModified( true );
						fileInfo.setCountdownTickCount( now );
					}
					fileInfo.update( finddata );
					fileInfo.setExist( true );
					fileInfo.setDeletePending( false );
				}
				else {
					CFileInfo fileInfo( finddata, v_enableEvent, true );
					fileInfo.setDeletePending( false );
					fileInfo.setCountdownTickCount( now );
					fileInfoMap_.insert( CFileInfoMap::value_type( absolutePath, fileInfo ) );
				}
			}
		}
		if( ! ::FindNextFile( findHandle, &finddata ) ) {
			::FindClose( findHandle );
			break;
		}
	}

	return true;
}

void CWatchThread::run()
{
	// t@CɎgpfBNgpX̍쐬
	const tstring baseDir = tstringuty::ensureEndsWith( settingInfo_.getWatchDir(), tstring("\\") );


	// Xg̏(ʒmȂ)
	EnumFile( false );

	DelaySignal enumerationSignal( settingInfo_.getDelaySignalSpan() );
	PassiveTimer forceEnumerationTimer( settingInfo_.getForceInterval() * 1000, ::GetTickCount(), settingInfo_.getForceInterval() > 0 );
	PassiveTimer garbageCollectTimer( 60 * 1000 ); // 60b
	PassiveTimer notifyHandleExpiryTimer( settingInfo_.getDirNotificationAPIExpirySpan() * 1000, 0, settingInfo_.getDirNotificationAPIExpirySpan() > 0 );

	// t@CύXʒm[v̊Jn
	HANDLE findHandle = INVALID_HANDLE_VALUE;
	for(;;) {
		// ݎ̎擾
		const DWORD now = ::GetTickCount();

		// q[v̈k(s)
		if( garbageCollectTimer.isSignaled( now ) ) {
			garbageCollectTimer.reset( now );
			sweepDeletedEntries();
			_heapmin();
		}

		// ʒmnh̍쐬
		if( settingInfo_.isUsingDirNotificationAPI() &&
			( notifyHandleExpiryTimer.isSignaled( now ) ||
			  findHandle == INVALID_HANDLE_VALUE
			) )
		{
			::OutputDebugString( _TEXT("Create: FindFirstCHangeNotificationHandle\n") );
			findHandle = ::FindFirstChangeNotification(
				settingInfo_.getWatchDir().c_str(),
				settingInfo_.getMaxDepth() > 0 ? TRUE : FALSE,
				FILE_NOTIFY_CHANGE_FILE_NAME |
				FILE_NOTIFY_CHANGE_DIR_NAME |
				FILE_NOTIFY_CHANGE_LAST_WRITE |
				FILE_NOTIFY_CHANGE_SIZE |
				FILE_NOTIFY_CHANGE_ATTRIBUTES
				);
			if( findHandle != INVALID_HANDLE_VALUE ) {
				notifyHandleExpiryTimer.reset( now );
			}
			//if( findHandle == INVALID_HANDLE_VALUE ) {
			//	DWORD errcode = GetLastError();
			//}
		}

		// ANV̎s
		pActionInvoker_->sweepTerminatedProcess();
		const DWORD writeWaitLimit = (DWORD)( settingInfo_.getWaitWrite() * 100 );
		for( CFileInfoMap::iterator ite = fileInfoMap_.begin();
			ite != fileInfoMap_.end();
			++ite)
		{
			if( settingInfo_.getMaxProcess() != 0 && pActionInvoker_->sweepTerminatedProcess() >= settingInfo_.getMaxProcess() ) {
				// vZXEɒB̂ŌssȂB
				break;
			}
			const tstring& absolutePath = ite->first;
			CFileInfo& fileInfo = ite->second;
			if( fileInfo.isDetectModified() ) {
				const DWORD span = now - fileInfo.getCountdownTickCount();
				if( span >= writeWaitLimit ) {
					DWORD nFileSizeLow;
					if( ! getFileSize( absolutePath, &nFileSizeLow ) ||	( fileInfo.getSize() != nFileSizeLow ) ) {
						// t@CJȂAt@CTCYύXĂ΃JEĝ蒼
						fileInfo.setSize( nFileSizeLow );
						fileInfo.setCountdownTickCount( now );
					}
					else{
						fileInfo.setDetectModified( false );
						::OutputDebugString( _TEXT("do action\n") );
						pActionInvoker_->createProcess( absolutePath, fileInfo );
					}
				}
			}
		}

		// Cxg擾
		HANDLE handles[] = { hStopEvent_, NULL };
		int handleCount = 1;
		if( findHandle != INVALID_HANDLE_VALUE ) {
			handles[ 1 ] = findHandle;
			handleCount = 2;
		}

		const DWORD ret = ::WaitForMultipleObjects( handleCount, handles, FALSE, 10 ); // bZ[W10msȑOɖ߂\B
		if( ret == WAIT_OBJECT_0 || ret == WAIT_ABANDONED_0 ) {
			// ~Cxg
			break;
		}
		else if( ret == WAIT_OBJECT_0 + 1 || ret == WAIT_ABANDONED_0 + 1 ) {
			enumerationSignal.setSignalDelay( now );
			::OutputDebugString( _TEXT("signal: dirchange notification\n") );
			if( ! ::FindNextChangeNotification( findHandle ) ){
				break;
			}
		}

		if( forceEnumerationTimer.isSignaled( now ) ) {
			// XVw肳ĂAŌɌĂw莞Ԃo߂Ă邽ߋsB
			forceEnumerationTimer.reset( now );
			enumerationSignal.setSignalImmediate();
			::OutputDebugString( _TEXT("signal: force interval\n") );
		}

		if( enumerationSignal.isSignaled( now ) ) {
			enumerationSignal.clear();
			forceEnumerationTimer.reset( now );
			::OutputDebugString( _TEXT("begin Enumeration\n") );
			EnumFile(true);
		}
	}
	if(findHandle != INVALID_HANDLE_VALUE){
		::FindCloseChangeNotification( findHandle );
	}

	fileInfoMap_.clear();
}

unsigned __stdcall CWatchThread::FWatchThread( void* pParam )
{
	CWatchThread* me = (CWatchThread*) pParam;
	if( SUCCEEDED( ::CoInitializeEx( NULL, COINIT_APARTMENTTHREADED ) ) ) {
		me->run();
		::CoUninitialize();
	}
	else {
		pApp->showMessageBox( MB_ICONERROR | MB_OK, NULL, IDS_MAINPROC_COINITFAILED );
	}
	return 0;
}
