#include "stdafx.hpp"
#include "resource.h"

#include "FWatchApp.hpp"

#include "Utility.hpp"
#include "tstringUty.hpp"

#include "SettingInfoFactory.hpp"
#include "WatchThreadGroup.hpp"

#include <assert.h>

#include <fstream>
#include <exception>

//////////////////////////////////////////////

CWatchThreadGroup::CWatchThreadGroup( const bool v_dryrun )
	: modified_( false )
	, dryrun_( v_dryrun )
{
}

void CWatchThreadGroup::ReleaseAllThreads()
{
	// ~߂ׂẴXbhɑM(ҋ@͂ȂB)
	for( CWatchThreadList::iterator ite = watchList_.begin();
		ite != watchList_.end();
		++ite )
	{
		CWatchThread* pWatchThread = *ite;
		pWatchThread->StopNoWait();
	}

	// ׂẴXbhjB(X̃Xbh̔jҋ@)
	for( CWatchThreadList::iterator ite = watchList_.begin();
		ite != watchList_.end();
		++ite )
	{
		CWatchThread* pWatchThread = *ite;
		delete pWatchThread;
	}

	watchList_.clear();
	modified_ = false;
}

CWatchThreadGroup::~CWatchThreadGroup()
{
	ReleaseAllThreads();
}

bool CWatchThreadGroup::isDryrRun() const
{
	return dryrun_;
}

namespace
{
	class SettingInfoFactoryAdaptor
	{
	private:
		CSettingInfoFactory& factory_;
	public:
		SettingInfoFactoryAdaptor( CSettingInfoFactory& v_factory )
			: factory_( v_factory )
		{
		}

		void setWatchDir( const tstring& arg ) {
			factory_.setWatchDir( arg );
		}

		void setWatchFile( const tstring& arg ) {
			factory_.setWatchFile( arg );
		}

		void setAction( const tstring& arg ) {
			factory_.setAction( arg );
		}

		void setAppName( const tstring& arg ) {
			factory_.setAppName( arg );
		}

		void setParam( const tstring& arg ) {
			factory_.setParam( arg );
		}

		void setMaxProcess( const tstring& arg ) {
			factory_.setMaxProcess( tstringuty::atoi( arg ) );
		}

		void setShowWindow( const tstring& arg ) {
			factory_.setShowWindow( ShowWindowType::valueOf( tstringuty::atoi( arg ) ) );
		}

		void setWaitWrite( const tstring& arg ) {
			factory_.setWaitWrite( tstringuty::atoi( arg ) );
		}

		void setTolerance( const tstring& arg ) {
			factory_.setTolerance( tstringuty::atoi( arg ) );
		}

		void setLog( const tstring& arg ) {
			factory_.setLog( arg );
		}

		void setMaxDepth( const tstring& arg ) {
			factory_.setMaxDepth( tstringuty::atoi( arg ) );
		}

		void setUsingDirNotificationAPI( const tstring& arg ) {
			factory_.setUsingDirNotificationAPI( tstringuty::atoi( arg ) != 0 );
		}

		void setForceInterval( const tstring& arg ) {
			factory_.setForceInterval( tstringuty::atoi( arg ) );
		}

		void setDeletePending( const tstring& arg ) {
			factory_.setDeletePending( tstringuty::atoi( arg ) );
		}

		void setDirNotificationAPIExpirySpan( const tstring& arg ) {
			factory_.setDirNotificationAPIExpirySpan( tstringuty::atoi( arg ) );
		}

		void setDirNotificationAPIRetryInterval( const tstring& arg ) {
			factory_.setDirNotificationAPIRetryInterval( tstringuty::atoi( arg ) );
		}

		void setDelaySignalSpan( const tstring& arg ) {
			factory_.setDelaySignalSpan( tstringuty::atoi( arg ) );
		}

		void setAppCurrentDir( const tstring& arg ) {
			factory_.setAppCurrentDir( arg );
		}

		typedef void (SettingInfoFactoryAdaptor::*adaptorFunc)( const tstring& );

		static const SettingInfoFactoryAdaptor::adaptorFunc inputListFunctions[]; 
	};


	const SettingInfoFactoryAdaptor::adaptorFunc SettingInfoFactoryAdaptor::inputListFunctions[] = {
		SettingInfoFactoryAdaptor::setWatchDir,
		SettingInfoFactoryAdaptor::setWatchFile,
		SettingInfoFactoryAdaptor::setAction,
		SettingInfoFactoryAdaptor::setAppName,
		SettingInfoFactoryAdaptor::setParam,
		SettingInfoFactoryAdaptor::setMaxProcess,
		SettingInfoFactoryAdaptor::setShowWindow,
		SettingInfoFactoryAdaptor::setWaitWrite,
		SettingInfoFactoryAdaptor::setTolerance,
		SettingInfoFactoryAdaptor::setDeletePending,
		SettingInfoFactoryAdaptor::setLog,
		SettingInfoFactoryAdaptor::setMaxDepth,
		SettingInfoFactoryAdaptor::setUsingDirNotificationAPI,
		SettingInfoFactoryAdaptor::setForceInterval,
		SettingInfoFactoryAdaptor::setDirNotificationAPIExpirySpan,
		SettingInfoFactoryAdaptor::setDirNotificationAPIRetryInterval,
		SettingInfoFactoryAdaptor::setDelaySignalSpan,
		SettingInfoFactoryAdaptor::setAppCurrentDir,
		NULL
	};
}


bool CWatchThreadGroup::LoadInfData( const tstring& InfPath )
{
	if( InfPath.empty() ) {
		return false;
	}

	ReleaseAllThreads();

	bool succeeded = false;
	try{
		std::ifstream fin;
		fin.open( InfPath.c_str() );
		if( ! fin.is_open() ) {
			return false;
		}

		const CSettingInfo init;

		tstring line;
		while( std::getline( fin, line ) ) {

			CSettingInfoFactory factory( init );
			SettingInfoFactoryAdaptor adaptor( factory );

			int col = 0;
			tstring::size_type lastpos = 0;
			for(;;) {
				const tstring::size_type pos = line.find( '\t', lastpos );
				tstring arg;
				if( pos != tstring::npos ) {
					arg = line.substr( lastpos, pos - lastpos );
				}
				else {
					arg = line.substr( lastpos );
				}
				
				SettingInfoFactoryAdaptor::adaptorFunc func = SettingInfoFactoryAdaptor::inputListFunctions[ col ];
				if( func == NULL ) {
					break;
				}

				(adaptor.*func)( arg );

				col++;
				lastpos = pos + 1;
				if( pos == tstring::npos ) {
					break;
				}
			}

			append( factory.create() );
		}
		fin.close();
		succeeded = true;
	}
	catch( const std::exception& v_exception ) {
		pApp->showMessageBox( MB_ICONERROR | MB_OK, NULL, IDS_MAINPROC_INFLOADFAILED, v_exception.what() );
	}

	modified_ = false;
	return succeeded;
}

bool CWatchThreadGroup::isModified() const
{
	return modified_;
}

bool CWatchThreadGroup::SaveInfData( const tstring& InfPath, bool v_bForce ) const
{
	if( InfPath.empty() ) {
		return false;
	}

	if( ! isModified() && ! v_bForce ) {
		return true;
	}

	bool succeeded = false;
	try{
		std::ofstream fout;
		fout.exceptions( std::ios::failbit | std::ios::badbit );
		fout.open( InfPath.c_str() );
		if( ! fout.is_open() ) {
			return false;
		}
		for( CWatchThreadList::const_iterator ite = watchList_.begin();
			ite != watchList_.end();
			++ite )
		{
			const CWatchThread* pWatchThread = *ite;
			const CSettingInfo& settingInfo = pWatchThread->getSettingInfo();

			fout << settingInfo.getWatchDir()	<< "\t";
			fout << settingInfo.getWatchFile()	<< "\t";
			fout << settingInfo.getAction()		<< "\t";
			fout << settingInfo.getAppName()	<< "\t";
			fout << settingInfo.getParam()		<< "\t";
			fout << settingInfo.getMaxProcess()	<< "\t";
			fout << settingInfo.getShowWindow()	<< "\t";
			fout << settingInfo.getWaitWrite()	<< "\t";
			fout << settingInfo.getTolerance()	<< "\t";
			fout << settingInfo.getDeletePending() << "\t";
			fout << settingInfo.getLog()		<< "\t";
			fout << settingInfo.getMaxDepth()	<< "\t";
			fout << ( settingInfo.isUsingDirNotificationAPI() ? 1 : 0 )	<< "\t";
			fout << settingInfo.getForceInterval()						<< "\t";
			fout << settingInfo.getDirNotificationAPIExpirySpan()		<< "\t";
			fout << settingInfo.getDirNotificationAPIRetryInterval()	<< "\t";
			fout << settingInfo.getDelaySignalSpan() << "\t";
			fout << settingInfo.getAppCurrentDir() << std::endl;
		}

		modified_ = false;
		succeeded = true;
	}
	catch( const std::exception& v_exception ) {
		pApp->showMessageBox( MB_ICONERROR | MB_OK, NULL, IDS_MAINPROC_INFSAVEFAILED, v_exception.what() );
	}

	return succeeded;
}

unsigned int CWatchThreadGroup::size() const
{
	return (unsigned int) watchList_.size();
}

const CSettingInfo& CWatchThreadGroup::getInfo( unsigned int idx ) const
{
	//required:
	assert( idx >= 0 && idx < watchList_.size() && "݂ȂACeɂ̓ANZXł܂B" );
	
	//do:
	return watchList_[ idx ]->getSettingInfo();
}

bool CWatchThreadGroup::append( const CSettingInfo& v_settingInfo )
{
	CWatchThread* pWatchThread = new CWatchThread( v_settingInfo );
	watchList_.push_back( pWatchThread );
	if( ! dryrun_ ) {
		pWatchThread->Start();
	}
	modified_ = true;
	return true;
}

bool CWatchThreadGroup::replace( unsigned int idx, const CSettingInfo& v_settingInfo )
{
	//required:
	assert( idx >= 0 && idx < watchList_.size() && "݂ȂACeɂ̓ANZXł܂B" );
	
	//do:

	delete watchList_[ idx ];
	CWatchThread *const pWatchThread = new CWatchThread( v_settingInfo );
	watchList_[ idx ] = pWatchThread;
	if( ! dryrun_ ) {
		pWatchThread->Start();
	}
	modified_ = true;
	return true;
}

bool CWatchThreadGroup::erase(unsigned int idx)
{
	//required:
	assert( idx >= 0 && idx < watchList_.size() && "͈͊OłB" );

	//do:
	CWatchThreadList::iterator ite = watchList_.begin() + idx;
	delete *ite;
	watchList_.erase( ite );
	modified_ = true;
	return true;
}

bool CWatchThreadGroup::isRunning() const
{
	bool ret = false;
	for( CWatchThreadList::const_iterator ite = watchList_.begin();
		! ret && ite != watchList_.end();
		++ite )
	{
		const CWatchThread* pWatchThread = *ite;
		ret |= pWatchThread->isRunning();
	}
	return ret;
}
