// UnitTest.cpp : R\[ AvP[ṼGg |Cg`܂B
//

#include "stdafx.hpp"

#include <assert.h>
#include <vector>
#include <iterator>

#include "..\FWatch\Utility.hpp"
#include "..\FWatch\\tstringUty.hpp"
#include "..\FWatch\StringExpander.hpp"
#include "..\FWatch\SplitPath.hpp"
#include "..\FWatch\PatternMatch.hpp"

void testPropertyStringExpander();
void testIsMatch();
void testSplitPath();
void testEnsureEndsWith();
void testPatternMatch();

int _tmain(int argc, _TCHAR* argv[])
{
	DEBUGBLOCK(
		_CrtSetDbgFlag( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF );
	);

	std::vector< tstring > buf;
	split( "hello|world|abc", '|', std::insert_iterator<std::vector<tstring> >(buf, buf.end() ) );
	int cnt = 0;
	for( std::vector<tstring>::iterator ite = buf.begin();
		ite != buf.end();
		++ite)
	{
		cnt++;
		_tprintf( _TEXT("%d=%s\n"), cnt, (*ite).c_str() );
	}

	testPropertyStringExpander();
	testIsMatch();
	testSplitPath();
	testEnsureEndsWith();
	testPatternMatch();

	return 0;
}

void testPropertyStringExpander()
{
	// eXgp^[

	PropertyStringExpander expander;
	expander.addProperty( _TEXT("ABC"), _TEXT("123") );
	
	assert( expander.expandString( _TEXT("") ) == _TEXT("") );
	assert( expander.expandString( _TEXT("123 456") ) == _TEXT("123 456") );
	assert( expander.expandString( _TEXT("%ABC%") ) == _TEXT("123") );
	assert( expander.expandString( _TEXT("%abc%") ) == _TEXT("123") );
	assert( expander.expandString( _TEXT("%DEF%") ) == _TEXT("%DEF%") );
	assert( expander.expandString( _TEXT("%%ABC%%") ) == _TEXT("%123%") );
	assert( expander.expandString( _TEXT("%%DEF%%") ) == _TEXT("%%DEF%%") );
	assert( expander.expandString( _TEXT("%ABC%%%DEF%%") ) == _TEXT("123%%DEF%%") );
	assert( expander.expandString( _TEXT("%ABC%% %DEF%%") ) == _TEXT("123% %DEF%%") );
	assert( expander.expandString( _TEXT("%ABC%% %DEF%% ") ) == _TEXT("123% %DEF%% ") );

	assert( expander.expandString( _TEXT("%1 %2 %* -path=\"%ABC%\"" )) == _TEXT("%1 %2 %* -path=\"123\"") );
}

void testIsMatch()
{
	// eXgp^[

	assert( IsMatch( _TEXT(""), _TEXT("") ) );
	assert( ! IsMatch( _TEXT(""), _TEXT("?") ) );
	assert( IsMatch( _TEXT(""), _TEXT("*") ) );

	assert( IsMatch( _TEXT("A"), _TEXT("A") ) );
	assert( IsMatch( _TEXT("A"), _TEXT("?") ) );
	assert( IsMatch( _TEXT("AB"), _TEXT("?B") ) );
	assert( IsMatch( _TEXT("AB"), _TEXT("A?") ) );
	assert( ! IsMatch( _TEXT("AB"), _TEXT("??B") ) );
	assert( ! IsMatch( _TEXT("AB"), _TEXT("A?C") ) );

	assert( IsMatch( _TEXT("ABC"), _TEXT("ABC") ) );
	assert( IsMatch( _TEXT("ABC"), _TEXT("ABC*") ) );
	assert( IsMatch( _TEXT("ABCD"), _TEXT("ABC*") ) );
	assert( IsMatch( _TEXT("ABCABC"), _TEXT("ABC*") ) );

	assert( IsMatch( _TEXT("ABC ABC"), _TEXT("ABC*") ) );
	assert( IsMatch( _TEXT("ABC DEF"), _TEXT("ABC*DEF") ) );
	assert( IsMatch( _TEXT("ABC DEF DEF"), _TEXT("ABC*DEF") ) );
	assert( IsMatch( _TEXT("ABC"), _TEXT("*ABC") ) );
	assert( IsMatch( _TEXT("DEFABC"), _TEXT("*ABC") ) );
	assert( IsMatch( _TEXT("ABCDEFABC"), _TEXT("*DEF*") ) );
	assert( IsMatch( _TEXT("ABCDABC"), _TEXT("*ABC") ) );

	assert( ! IsMatch( _TEXT("ABCDABCD"), _TEXT("*ABC") ) );

	assert( IsMatch( _TEXT("a.txt"), _TEXT("*.txt") ) );
	assert( IsMatch( _TEXT("a.b.txt"), _TEXT("*.txt") ) );
	assert( IsMatch( _TEXT("a.b.txt"), _TEXT("*.b.*") ) );

	assert( ! IsMatch( _TEXT("a.txt.bin"), _TEXT("*.txt") ) );
	assert( ! IsMatch( _TEXT("a.txtx"), _TEXT("*.txt") ) );
	assert( ! IsMatch( _TEXT("a.c.d"), _TEXT("*.b.*") ) );

	assert( ! IsMatch( _TEXT("\"), _TEXT("?\\*") ) );
	assert( ! IsMatch( _TEXT("\"), _TEXT("*\\*") ) );
	assert( ! IsMatch( _TEXT("\\"), _TEXT("?\\*") ) );
	assert( ! IsMatch( _TEXT("\\"), _TEXT("*\\*") ) );
	assert( IsMatch( _TEXT("\\\\"), _TEXT("?\\*") ) );
	assert( IsMatch( _TEXT("\\\\"), _TEXT("*\\\\") ) );
}

void testSplitPath()
{
	SplitPath splitPath;

	assert( splitPath.getPath().empty() );
	assert( splitPath.getDirectory().empty() );
	assert( splitPath.getFile().empty() );
	assert( splitPath.getFileExtension().empty() );
	assert( splitPath.getFileNoExtension().empty() );

	splitPath.setPath( _TEXT("abc") );
	assert( splitPath.getPath() == _TEXT("abc") );
	assert( splitPath.getDirectory().empty() );
	assert( splitPath.getFile() == _TEXT("abc") );
	assert( splitPath.getFileExtension().empty() );
	assert( splitPath.getFileNoExtension() == _TEXT("abc") );

	splitPath.setPath( _TEXT("C:\\abc\\") );
	assert( splitPath.getPath() == _TEXT("C:\\abc\\") );
	assert( splitPath.getDirectory() == _TEXT("C:\\abc\\") );
	assert( splitPath.getFile().empty() );
	assert( splitPath.getFileExtension().empty() );
	assert( splitPath.getFileNoExtension().empty() );

	splitPath.setPath( _TEXT("\\\\HOST\\abc\\") );
	assert( splitPath.getPath() == _TEXT("\\\\HOST\\abc\\") );
	assert( splitPath.getDirectory() == _TEXT("\\\\HOST\\abc\\") );
	assert( splitPath.getFile().empty() );
	assert( splitPath.getFileExtension().empty() );
	assert( splitPath.getFileNoExtension().empty() );

	splitPath.setDirectory( _TEXT("D:\\xyz") );
	assert( splitPath.getPath() == _TEXT("D:\\xyz\\") );
	assert( splitPath.getDirectory() == _TEXT("D:\\xyz\\") );
	assert( splitPath.getFile().empty() );
	assert( splitPath.getFileExtension().empty() );
	assert( splitPath.getFileNoExtension().empty() );

	splitPath.setFile( _TEXT("test") );
	assert( splitPath.getPath() == _TEXT("D:\\xyz\\test") );
	assert( splitPath.getDirectory() == _TEXT("D:\\xyz\\") );
	assert( splitPath.getFile() == _TEXT("test") );
	assert( splitPath.getFileExtension().empty() );
	assert( splitPath.getFileNoExtension() == _TEXT("test") );

	splitPath.setFile( _TEXT("test2.") );
	assert( splitPath.getPath() == _TEXT("D:\\xyz\\test2") );
	assert( splitPath.getDirectory() == _TEXT("D:\\xyz\\") );
	assert( splitPath.getFile() == _TEXT("test2") );
	assert( splitPath.getFileExtension().empty() );
	assert( splitPath.getFileNoExtension() == _TEXT("test2") );

	splitPath.setFile( _TEXT("test3.txt") );
	assert( splitPath.getPath() == _TEXT("D:\\xyz\\test3.txt") );
	assert( splitPath.getDirectory() == _TEXT("D:\\xyz\\") );
	assert( splitPath.getFile() == _TEXT("test3.txt") );
	assert( splitPath.getFileExtension() == _TEXT("txt") );
	assert( splitPath.getFileNoExtension() == _TEXT("test3") );

	splitPath.setPath( _TEXT("D:\\ABC.TXT\\DEF.XYZ\\test4.dat") );
	assert( splitPath.getPath() == _TEXT("D:\\ABC.TXT\\DEF.XYZ\\test4.dat") );
	assert( splitPath.getDirectory() == _TEXT("D:\\ABC.TXT\\DEF.XYZ\\") );
	assert( splitPath.getFile() == _TEXT("test4.dat") );
	assert( splitPath.getFileExtension() == _TEXT("dat") );
	assert( splitPath.getFileNoExtension() == _TEXT("test4") );

	splitPath.setPath( _TEXT("D:\\ABC.TXT\\DEF.XYZ\\test5.dat.txt") );
	assert( splitPath.getPath() == _TEXT("D:\\ABC.TXT\\DEF.XYZ\\test5.dat.txt") );
	assert( splitPath.getDirectory() == _TEXT("D:\\ABC.TXT\\DEF.XYZ\\") );
	assert( splitPath.getFile() == _TEXT("test5.dat.txt") );
	assert( splitPath.getFileExtension() == _TEXT("txt") );
	assert( splitPath.getFileNoExtension() == _TEXT("test5.dat") );

	splitPath.setPath( _TEXT("\\\\HOST\\ABC.TXT\\DEF.XYZ\\test6.dat.txt") );
	assert( splitPath.getPath() == _TEXT("\\\\HOST\\ABC.TXT\\DEF.XYZ\\test6.dat.txt") );
	assert( splitPath.getDirectory() == _TEXT("\\\\HOST\\ABC.TXT\\DEF.XYZ\\") );
	assert( splitPath.getFile() == _TEXT("test6.dat.txt") );
	assert( splitPath.getFileExtension() == _TEXT("txt") );
	assert( splitPath.getFileNoExtension() == _TEXT("test6.dat") );

	splitPath.setFileExtension( _TEXT("hlp") );
	assert( splitPath.getPath() == _TEXT("\\\\HOST\\ABC.TXT\\DEF.XYZ\\test6.dat.hlp") );
	assert( splitPath.getDirectory() == _TEXT("\\\\HOST\\ABC.TXT\\DEF.XYZ\\") );
	assert( splitPath.getFile() == _TEXT("test6.dat.hlp") );
	assert( splitPath.getFileExtension() == _TEXT("hlp") );
	assert( splitPath.getFileNoExtension() == _TEXT("test6.dat") );

}

void testEnsureEndsWith()
{
	assert( tstringuty::getCharCount( _TEXT("" ) ) == 0 );
	assert( tstringuty::getCharCount( _TEXT("A" ) ) == 1 );
	assert( tstringuty::getCharCount( _TEXT("AA" ) ) == 2 );
	assert( tstringuty::getCharCount( _TEXT("AA\" ) ) == 3 );
	assert( tstringuty::getCharCount( _TEXT("\\" ) ) == 2 );
	assert( tstringuty::getCharCount( _TEXT("\A\" ) ) == 3 );
	assert( tstringuty::getCharCount( _TEXT("\A\A" ) ) == 4 );
	assert( tstringuty::getCharCount( _TEXT("A\\A\\A" ) ) == 7 );

	assert( tstringuty::ensureEndsWith( _TEXT(""), _TEXT("\\") ) == _TEXT("\\") );
	assert( tstringuty::ensureEndsWith( _TEXT("\\"), _TEXT("\\") ) == _TEXT("\\") );
	assert( tstringuty::ensureEndsWith( _TEXT("ABC"), _TEXT("\\") ) == _TEXT("ABC\\") );
	assert( tstringuty::ensureEndsWith( _TEXT("ABC\\"), _TEXT("\\") ) == _TEXT("ABC\\") );
	assert( tstringuty::ensureEndsWith( _TEXT("\"), _TEXT("\\") ) == _TEXT("\\\") );
	assert( tstringuty::ensureEndsWith( _TEXT("\\\"), _TEXT("\\") ) == _TEXT("\\\") );
}

void testPatternMatch()
{
	PatternMatchCollection patterns1( true ); // ׂă}b`OK
	assert( patterns1.isMatch( "any" ) );
	
	patterns1.addPattern( SimplePatternMatch( "*.txt" ) );
	assert( patterns1.isMatch( "abc123.txt" ) );
	assert( ! patterns1.isMatch( "abc123.dat" ) );

	patterns1.addPattern( SimplePatternMatch( "*123.txt" ) );
	assert( patterns1.isMatch( "abc123.txt" ) );
	assert( ! patterns1.isMatch( "abc123.dat" ) );

	patterns1.addPattern( SimplePatternMatch( "*123.dat" ) );
	assert( ! patterns1.isMatch( "abc123.txt" ) );
	assert( ! patterns1.isMatch( "abc123.dat" ) );

	PatternMatchCollection patterns2( false ); // 1ł}b`OK
	assert( ! patterns2.isMatch( "any" ) );
	
	patterns2.addPattern( SimplePatternMatch( "*.txt" ) );
	assert( patterns2.isMatch( "abc123.txt" ) );
	assert( ! patterns2.isMatch( "abc123.dat" ) );

	patterns2.addPattern( SimplePatternMatch( "*123.dat" ) );
	assert( patterns2.isMatch( "abc123.txt" ) );
	assert( patterns2.isMatch( "abc123.dat" ) );

}

